
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function Overlay()
{
	this.control   = null;
	this.statusbar = null;
	this.popupset  = null;
	this.menupopup = null;
	this.observer  = null;

// -----------------------------------------------------------------------------

	this.openDialog = function(dialogName)
	{
		var url      = null;
		var name     = null;
		var features = null;
		switch (dialogName)
		{
			case "profile":
				url      = "chrome://prefswitch/content/profiledialog.xul";
				name     = "profiledialog";
				features = "modal";
				break;
			case "switch":
				url      = "chrome://prefswitch/content/switchdialog.xul";
				name     = "switchdialog";
				features = "";
		}
		window.openDialog(url, name, features, this.control);
	};

// -----------------------------------------------------------------------------

	this.handleEvent = function(event)
	{
		switch (event.type)
		{
			case "load":
				this.control   = new Control(this);
				this.statusbar = document.getElementById("status-bar");
				this.popupset  = document.getElementById("popupset");
				this.menupopup = document.getElementById("profile-menupopup");
				this.updateProfiles();
				this.updateAll();
				this.observer = new Observer(this.control, this.control.UPDATE_ALL);
				this.observer.register();
				break;
			case "unload":
				var statusbarpanel = this.statusbar.firstChild;
				while (statusbarpanel)
				{
					if (statusbarpanel.getAttribute("class") == "initial") statusbarpanel.o.unregister();
					statusbarpanel = statusbarpanel.nextSibling;
				}
		}
	};

	this.updateAll = function()
	{
		this.removeStatusbarpanels();
		this.removeMenupopups();
		this.addStatusbarpanels();
	};

	this.removeStatusbarpanels = function()
	{
		var statusbarpanel = this.statusbar.firstChild;
		while (statusbarpanel)
		{
			var next = statusbarpanel.nextSibling;
			if (statusbarpanel.getAttribute("class") == "initial")
			{
				statusbarpanel.ob.unregister();
				this.statusbar.removeChild(statusbarpanel);
			}
			statusbarpanel = next;
		}
	}

	this.removeMenupopups = function()
	{
		var popup = this.popupset.firstChild;
		while (popup)
		{
			var next = popup.nextSibling;
			if (popup.tagName == "popup") this.popupset.removeChild(popup);
			popup = next;
		}
	}

	this.addStatusbarpanels = function()
	{
		var aSwitch = this.control.getSwitches();
		while (aSwitch.length > 0)
		{
			var tSwitch = aSwitch.shift();
			if (tSwitch.isHidden()) continue;
			// CREATE STATUSBARPANEL
			statusbarpanel = document.createElement("statusbarpanel");
			statusbarpanel.sw = tSwitch;
			statusbarpanel.ob = new Observer(this.control, this.control.UPDATE_ONE, tSwitch.getPrefRoot(), tSwitch.getPreference(), statusbarpanel);
			statusbarpanel.ob.register();
			statusbarpanel.setAttribute("class", "initial");
			statusbarpanel.setAttribute("label", tSwitch.initial);
			statusbarpanel.setAttribute("context", "menupopup");
			this.updateOne(statusbarpanel);
			// CREATE POPUP MENU OR APPEND CLICK EVENT
			if (tSwitch.type == this.control.BOOLEAN) statusbarpanel.setAttribute("onclick", "overlay.onclick(event, this)");
			else
			{
				var popup = document.createElement("popup");
				var id    = tSwitch.name; // it's not unique!
				popup.setAttribute("id", id);
				// ADD MENUITEMS
				for (var i = 0; i < tSwitch.states.length; i++)
				{
					var menuitem    = document.createElement("menuitem");
					var description = tSwitch.states[i][1];
					var value       = tSwitch.states[i][0];
					if (description != "") menuitem.setAttribute("label", description);
					else menuitem.setAttribute("label", value);
					menuitem.st = statusbarpanel;
					menuitem.value = value;
					menuitem.setAttribute("oncommand", "overlay.onclick(null, this.st, this.value)");
					popup.appendChild(menuitem);
				}
				this.popupset.appendChild(popup);
				statusbarpanel.setAttribute("popup", id);
			}
			this.statusbar.appendChild(statusbarpanel);
		}
	};

	this.updateOne = function(statusbarpanel)
	{
		var tSwitch = statusbarpanel.sw;
		var value   = this.control.getValue(tSwitch.preference, tSwitch.states[0][0], tSwitch.type);
		var tooltiptext = tSwitch.getTooltiptextByValue(value);
		if (tooltiptext != "") statusbarpanel.setAttribute("tooltiptext", tooltiptext);
		else statusbarpanel.setAttribute("tooltiptext", value);
		var color = tSwitch.getColorByValue(value);
		if (color != "") statusbarpanel.setAttribute("style", "color: " + color);
	};

	this.onclick = function(event, statusbarpanel, value)
	{
		if (event != null && event != this.control.LOAD_PROFILE && event.button > 0) return;
		var tSwitch = statusbarpanel.sw;
		if (tSwitch.type == this.control.BOOLEAN)
		{
			if (event == this.control.LOAD_PROFILE) value = value == "true";
			else value = !this.control.getValue(tSwitch.preference, tSwitch.states[0][0], tSwitch.type);
		}
		this.control.setValue(tSwitch.preference, value, tSwitch.type);
	};

	this.updateProfiles = function()
	{
		while (this.menupopup.firstChild.getAttribute("id") != "profile-separator")
			this.menupopup.removeChild(this.menupopup.firstChild);
		var aProfile = this.control.getProfiles();
		while (aProfile.length > 0)
		{
			var menuitem = document.createElement("menuitem");
			var profile  = aProfile.pop();
			menuitem.setAttribute("label", profile.name);
			menuitem.setAttribute("oncommand", "overlay.loadProfile(this)");
			menuitem.p = profile;
			this.menupopup.insertBefore(menuitem, this.menupopup.firstChild); // use profile-separator instead!
		}
	};

	this.loadProfile = function(menuitem)
	{
		var profile = menuitem.p;
		var statusbarpanel = this.statusbar.firstChild;
		while (statusbarpanel)
		{
			var tSwitch = statusbarpanel.sw;
			if (statusbarpanel.getAttribute("class") == "initial")
				this.onclick(this.control.LOAD_PROFILE, statusbarpanel, profile.features[tSwitch.name]);
			statusbarpanel = statusbarpanel.nextSibling;
		}
	};
}

// -----------------------------------------------------------------------------

var overlay = new Overlay();
window.addEventListener("load", overlay, false);
window.addEventListener("unload", overlay, false);
